/******************************************************************************
 * @file     APROM_main.c
 * @version  V1.00
 * @brief    FMC APROM IAP sample program run on APROM.
 *
 *
 * @copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#include <stdio.h>

#include "M480.h"
#include "isp_user.h"
#include "fmc_user.h"

typedef void (FUNC_PTR)(void);

extern uint32_t  loaderImage1Base, loaderImage1Limit;   /* symbol of image start and end */

#define FMC_BANK0_Addr 			0x00000000
#define FMC_BANK1_Addr 			0x00040000
#define BANK1_Version_CTL 	0x0004001c
#define BANK0_Version_CTL 	0x0000001c

void SYS_Init(void)
{
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable HXT clock */
    CLK_EnableXtalRC(CLK_PWRCTL_HXTEN_Msk);

    /* Wait for HXT clock ready */
    CLK_WaitClockReady(CLK_STATUS_HXTSTB_Msk);

    /* Switch HCLK clock source to HXT */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HXT,CLK_CLKDIV0_HCLK(1));

    /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(FREQ_192MHZ);

    /* Set both PCLK0 and PCLK1 as HCLK/2 */
    CLK->PCLKDIV = CLK_PCLKDIV_PCLK0DIV2 | CLK_PCLKDIV_PCLK1DIV2;

    /* Enable UART module clock */
    CLK_EnableModuleClock(UART0_MODULE);

    /* Select UART module clock source as HXT and UART module clock divider as 1 */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UART0SEL_HXT, CLK_CLKDIV0_UART0(1));

    /* Update System Core Clock */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART0 multi-function pins, RXD(PD.2) and TXD(PD.3) */
    SYS->GPD_MFPL &= ~(SYS_GPD_MFPL_PD2MFP_Msk | SYS_GPD_MFPL_PD3MFP_Msk);
    SYS->GPD_MFPL |= (SYS_GPD_MFPL_PD2MFP_UART0_RXD | SYS_GPD_MFPL_PD3MFP_UART0_TXD);

    /* Lock protected registers */
    SYS_LockReg();
}

void UART0_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Configure UART0 and set UART0 baud rate */
    UART_Open(UART0, 115200);
}

/*
 *  Set stack base address to SP register.
 */
#ifdef __ARMCC_VERSION                 /* for Keil compiler */
__asm __set_SP(uint32_t _sp)
{
    MSR MSP, r0
    BX lr
}
#endif

/**
  * @brief    Update APROM BANK1
  */
void UpdateAPROMBank0(void)
{

	  static uint32_t TotalLen;
	  static uint32_t StartAddressBank0, StartAddressBank1 = 1;
    uint16_t counter;
    uint32_t srclen;
    unsigned char *pSrc;
		uint32_t FMC_buff[16];
    //response = response_buff;                              //USB?
    //pSrc = buffer;                                         //
    srclen = 8;
    //lcmd = inpw(pSrc);
    //outpw(response + 4, 0);
    pSrc += 8;
    srclen = 8;
    //ReadData(Config0, Config0 + 16, (uint32_t *)(response + 8)); //read config
    //regcnf0 = *(uint32_t *)(response + 8);
	
    //StartAddress = 0;
	  StartAddressBank0 = 0;
		StartAddressBank1 = 0x40000;

    //StartAddress = inpw(pSrc);
    TotalLen = inpw(pSrc + 4);
    srclen = 32;
    //TotalLen_bak = TotalLen;
		EraseAP(StartAddressBank0, StartAddressBank0 + 0x30000);
		for(counter=0x4000; counter>0; counter--)
		{
	
				if (TotalLen < srclen) {
						srclen = TotalLen;//prevent last package from over writing
				}

				//TotalLen -= srclen;
				
				ReadData(StartAddressBank1, StartAddressBank1 + srclen, (uint32_t *)FMC_buff);
				WriteData(StartAddressBank0, StartAddressBank0 + srclen, (uint32_t *)FMC_buff);
				StartAddressBank0 += srclen;
				StartAddressBank1 += srclen;
				//LastDataLen =  srclen;

				//if (TotalLen == 0) {
						//lcksum = CalCheckSum(StartAddressBank1, TotalLen_bak);
						//outps(response + 8, lcksum);
				//}
		}
				
}

/**
  * @brief    Jump to APROM
  */
void JumpAPROMBank0(void)
{
		
		FUNC_PTR    *func;                 /* function pointer */

	  //Jump to Bank0: if(((*(uint32_t *)(0x00000000)) & 0xF0000000) == 0x20000000)
	  //Jump to Bank1: if(((*(uint32_t *)(0x00040000)) & 0xF0000000) == 0x20000000)		
		if(((*(uint32_t *)(0x00000000)) & 0xF0000000) == 0x20000000)		
		{
				/*  NOTE!
				 *     Before change VECMAP, user MUST disable all interrupts.
				 *     The following code CANNOT locate in address 0x0 ~ 0x200.
				 */
				/* FMC_SetVectorPageAddr(FMC_LDROM_BASE) */
				FMC->ISPCMD = FMC_ISPCMD_VECMAP;              /* ISP command */
				FMC->ISPADDR = 0x00000000;                /* Vector remap address */
				FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk;           /* Trigger ISP command */
				while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;  /* Wait for ISP command done. */
				

				/*
				 *  The reset handler address of an executable image is located at offset 0x4.
				 *  Thus, this sample get reset handler address of LDROM code from FMC_LDROM_BASE + 0x4.
				 */
				func = (FUNC_PTR *)*(uint32_t *)(FMC_APROM_BASE + 4);
				/*
				 *  The stack base address of an executable image is located at offset 0x0.
				 *  Thus, this sample get stack base address of LDROM code from FMC_LDROM_BASE + 0x0.
				 */
				__set_SP(*(uint32_t *)FMC_APROM_BASE);

				/*
				 *  Branch to the LDROM code's reset handler in way of function call.
				 */

				func();
		}
		else
				printf("\n\nFAIL!!\n");	  
	
}

/**
  * @brief    Load an image to specified flash address. The flash area must have been enabled by
  *           caller. For example, if caller want to program an image to LDROM, FMC_ENABLE_LD_UPDATE()
  *           must be called prior to calling this function.
  * @return   Image is successfully programmed or not.
  * @retval   0   Success.
  * @retval   -1  Program/verify failed.
  */

int main()
{
		uint32_t BANK0_Version;
		uint32_t BANK1_Version;
	
    SYS_Init();                        /* Init System, IP clock and multi-function I/O */

    UART0_Init();                      /* Initialize UART0 */

    /*---------------------------------------------------------------------------------------------------------*/
    /* SAMPLE CODE                                                                                             */
    /*---------------------------------------------------------------------------------------------------------*/



    SYS_UnlockReg();                   /* Unlock register lock protect */

    FMC_Open();                        /* Enable FMC ISP function */
		FMC_ENABLE_AP_UPDATE();

		BANK1_Version = FMC_Read(BANK1_Version_CTL);
		BANK0_Version = FMC_Read(BANK0_Version_CTL);

		if(BANK1_Version==0xFFFFFFFF)
		{
			JumpAPROMBank0();
			//printf("BANK1= %d, BANK0= %d\n", BANK1_Version, BANK0_Version);
		}
		else if (BANK1_Version > BANK0_Version)
		{
			UpdateAPROMBank0();
			//printf("BANK0= %d, BANK1= %d\n", BANK0_Version, BANK1_Version);
		}
		JumpAPROMBank0();
		
		while(1);


}

/*** (C) COPYRIGHT 2016 Nuvoton Technology Corp. ***/
